function [pred, confint, se, nu] = rspredict(y, X, x, alpha)
% RSPREDICT - Use linear regression to predict the response in a given point x, including a 1-alpha confidence interval. Version 0.1.4.
%
% The syntax is
% [pred, confint, se, nu] = rspredict(y, X, x, [alpha])
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
% x is a 1 by a row vector with the factor combination where one wants to predict.
% alpha is an optional parameter specifying the significance level to use for the confidence interval. If omitted, then alpha=0.05 is assumed.
%
% Return values
% pred is the predicted response.
% confint is a 1 by 2 row vector containing the end points of the symmetric (1-alpha)-confidence interval for the prediction. The width is zero if the model fits the data prefectly.
% se is the standard error of the prediction.
% nu is the number of degrees of freedom of the t-distribution (these come from the residual SS).
%
% References
% For background, see Section 2.5.3 in Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009.


% Copyright (C) 2015 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin < 3 || nargin > 4,
    error('Try help rspredict for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rspredict for instructions.');
end
if size(x,1) > 1,
    error('x has more than one row. Try help rspredict for instructions.');
end
if cond(X'*X) > 1e10,
    error('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

% Linear regression and prediction
b=X\y;
r=y-X*b;
[N,a]=size(X);
nu=N-a;
MSE=r'*r/nu;
pred=x*b;
se=sqrt(MSE*x*(X'*X)^-1*x');
if nargin == 3,
  alpha=0.05
end
quant=tinv(1-alpha/2,nu);
confint=pred+[-se*quant, se*quant];

end